<?php

/**
 * Class EnotMerchantAPI
 *
 */

class EnotMerchantAPI
{
    private string $_api_url;
    private string $_secretWord;

    /**
     * Constructor
     *
     * @param string $secretWord Your Terminal name
     */
    public function __construct(string $secretWord)
    {
        $this->_api_url = 'https://api.mivion.com/';
        $this->_secretWord = $secretWord;
    }

    /**
     * Builds a query string and call sendRequest method.
     * Could be used to custom API call method.
     *
     * @param string $path API method name
     * @param mixed $args query params
     * @param string $type
     * @return bool|string
     * @throws HttpException
     */
    public function buildQuery(string $path, mixed $args, string $type = 'get'): bool|string
    {
        $url = $this->_api_url;
        $url = $this->_combineUrl($url, $path);

        return $this->_sendRequest($url, $args, $type);
    }


    /**
     * Combines parts of URL. Simply gets all parameters and puts '/' between
     *
     * @return string
     */
    private function _combineUrl(): string
    {
        $args = func_get_args();
        $url = '';
        foreach ($args as $arg) {
            if (is_string($arg)) {
                if ($arg[strlen($arg) - 1] !== '/') $arg .= '/';
                $url .= $arg;
            }
        }
        return $url;
    }

    /**
     * Main method. Call API with params
     *
     * @param string $api_url API Url
     * @param mixed $args API params (could be array or string)
     * @param string $type
     * @return bool|string
     * @throws HttpException
     */
    private function _sendRequest(string $api_url, mixed $args, string $type): bool|string
    {
        if (is_array($args)) $args = json_encode($args);

        if ($curl = curl_init()) {
            curl_setopt($curl, CURLOPT_URL, $api_url);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            if ($type == 'get') {
                curl_setopt($curl, CURLOPT_POST, 0);
            } else {
                curl_setopt($curl, CURLOPT_POST, 1);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $args);
            }
            curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 30);
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json', 'x-api-key:' . $this->_secretWord
            ));

            $out = curl_exec($curl);
 
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

            curl_close($curl);
            if ($httpCode === 200) {
                return $out;
            } else {
                file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . " - sendRequest # enot error: {$out}\r\n", FILE_APPEND);
                return false;
            }

        } else {
            $message = "Can not create connection to {$api_url} with args {$args}";
            file_put_contents(DIR_LOGS . 'error.log', date('Y-m-d H:i:s') . " - sendRequest # enot: {$message}\r\n", FILE_APPEND);
            throw new HttpException(
                $message, 404
            );
        }
    }
}
